//------------------------------------------------------------------------------
//                      Introducción a los comandos de series de tiempo en Stata
//                                                        24 de septiembre, 2024
//------------------------------------------------------------------------------



// REPASO: COMPONENTES DE UNA SERIE DE TIEMPO
//------------------------------------------------------------------------------

clear all

// Generar años
set obs 120
generate year = ""
replace year = "2013" in 1/12
replace year = "2014" in 13/24
replace year = "2015" in 25/36
replace year = "2016" in 37/48
replace year = "2017" in 49/60
replace year = "2018" in 61/72
replace year = "2019" in 73/84
replace year = "2020" in 85/96
replace year = "2021" in 97/108
replace year = "2022" in 109/120

// Generar meses
generate month = _n
replace month = mod(month,12)
replace month = 12 if month == 0
tostring month, generate(str_month)
generate date = str_month + "/" + year
generate t = monthly(date,"MY")
format t %tm
drop year date str_month

// Tendencia lineal y multiplicativa
generate lin_trend = _n
generate mul_trend = _n^1.8

// Efectos estacionales
generate season = 0
replace season = 11 if month == 1
replace season = 13 if month == 2
replace season = 12 if month == 3
replace season = 15 if month == 4
replace season = 19 if month == 5
replace season = 23 if month == 6
replace season = 21 if month == 7
replace season = 28 if month == 8
replace season = 20 if month == 9
replace season = 17 if month == 10
replace season = 15 if month == 11
replace season = 12 if month == 12

// Estacionalidad lineal y multiplicativa
generate xt_linlin = lin_trend + season
generate xt_linmul = lin_trend*season
generate xt_mullin = mul_trend+season*50
generate xt_mulmul = mul_trend*season

// Ruido
generate xt_linlin_e = xt_linlin + rnormal()*50
generate xt_linmul_e = xt_linmul + rnormal()*500
generate xt_mullin_e = xt_mullin + rnormal()*1000
generate xt_mulmul_e = xt_mulmul + rnormal()*20000

// Declarar variable de tiempo
tsset t

// Gráficos
tsline lin_trend, ytitle("")
tsline mul_trend, ytitle("")
tsline xt_linlin, ytitle("")
tsline xt_linmul, ytitle("")
tsline xt_mullin, ytitle("")
tsline xt_mulmul, ytitle("")
tsline xt_linlin_e, ytitle("")
tsline xt_linmul_e, ytitle("")
tsline xt_mullin_e, ytitle("")
tsline xt_mulmul_e, ytitle("")


// SERIES UNIVARIADAS
//------------------------------------------------------------------------------

clear all

// Importar datos y etiquetar variables
import excel "ptmex.xls", sheet("Sheet1") firstrow
label variable lightrail "Pasajeros tren ligero (miles)"
label variable trolley "Pasajeros trolebus (miles)"
label variable rtp "Pasajeros RTP (milliones)"
label variable metro "Pasajeros Metro (milliones)"
label variable metrobus "Pasajeros Metrobus (miles)"
label variable suburbano "Pasajeros Suburbano"
label variable mexibus "Pasajeros Mexibus"
label variable mexicable "Pasajeros Mexicable"

rename date fecha

// String to date variable
generate t = monthly(fecha, "YM")
format t %tm
drop fecha

// Declarar variable de tiempo
tsset t, monthly

// Primer gráfico del Métro
tsline metro

// Eliminar la pandemia
keep if tin(1986m1, 2019m12)
tsline metro

//Mi interpretación inicial es que hay 3 periodos:
//- Un periodo más o menos estable hasta 2012
//- Luego un periodo breve de rápido crecimiento
//- Y luego otra vez un periodo estable a partir de 2014

// Hicimos una prueba sbcusum para revisar si hay otros periodos de
// inestabilidad, además del que me pareció más obvio
regress metro
estat sbcusum

// La inestabilidad al final de la serie es clara,
// pero sbcusum también detecta un poco de inestabilidad hacia el comienzo.
// Qué tal si suavizamos la serie y revisamos de nuevo? El suavizado elimina 
// el ruido para que podamos ver los otros componentes de la serie con mayor claridad.

// Suavizado de medias móviles (1 1 1)
tssmooth ma ma3_metro = metro, window(1 1 1) replace
tsline metro ma3_metro
tsline ma3_metro

// Este suavizado simétrico de ventana 3 ya revela otros detalles de la serie...
quietly regress ma3_metro
estat sbcusum // ...pero el gráfico sbcusum de la serie suavizada es muy similar al anterior.

// La estacionalidad es muy aparente en el periodo 2004-2006:
tsline metro ma3_metro if tin(2004m1, 2006m12)

// Ajuste estacional con suavizado (11 1)
tssmooth ma ma12_metro = metro, window(11 1) replace
tsline metro ma12_metro if tin(2004m1, 2006m12)
tsline metro ma12_metro

tsline ma12_metro // Podemos detectar varias historias más en esta serie
// desestacionalizada, como la Gran Recesión y la crisis económica de la
// segunda mitad de los 90s. Ambas generaron mucho desempleo, por lo que menos
// gente utilizaba el Metro para ir al trabajo.

quietly regress ma12_metro
estat sbcusum // La inestabilidad también es mucho más clara ahora en el gráfico sbcusum.

// Metrobus
tsline metrobus

// Tal vez parece que el número de pasajeros simplemente creció a un ritmo más o menos
// constante en estos 11 años entre 2008 y 2019, como la línea de tendencia
// que le dibujé ahí. Pero! Si le aplicamos aquí nuestro truco de desestacionalización
// para filtrar el ruido y la estacionalidad, podemos observar que el crecimiento es
// bastante irregular. Hay periodos de rápido crecimiento en el número de pasajeros cuando
// se inaugura una nueva línea de Metrobús, y luego periodos de estabilidad cuando no está
// inaugurando nada.
tssmooth ma ma12_metrobus = metrobus, window(11 1) replace
tsline metrobus ma12_metrobus
tsline ma12_metrobus

// La tendencia y la media no constante son un problema para muchos métodos de
// estimación que asumen estacionariedad. Esta serie evidentemente no es estacionaria
// pero le hacemos una prueba de raíz unitaria sólo para ver cómo se hace en Stata:
dfuller metrobus

// Y bueno ese resultado era obvio. Nuestra serie tiene una tendencia clara y no es
// estacionaria. Pero otra pregunta que nos podemos hacer es si las diferencias de esta serie, es decir los cambios mes contra mes en el número de pasajeros son estacionarios.
// Estamos hablando de esas fluctuaciones arriba-debajo que vemos alrededor de la
// tendencia en nuestra gráfica.

// Antes de revisar eso, veamos un gráfico de las diferencias
tsline D.metrobus

// Prueba de raíz unitaria
dfuller D.metrobus // Las primeras diferencias son estacionarias

tsline D.metrobus if tin(2008m1, 2019m12) // La media es constante, pero la
// varianza no. Retomaremos este punto después...

// Autocorrelogramas
ac D.metrobus
pac D.metrobus

// Selección de rezagos con arimasoc
arimasoc D.metrobus

// ARIMA
arima metrobus, arima(2, 1, 1)
estimates store arima21

// Prueba de efectos ARCH
quietly regress D.metrobus
estat archlm, lags(1)

// ARCH(1)
arch D.metrobus, ar(1) arch(1)
estimates store arch1

// Comparar arima21 y arch1
estimates stats arima21 arch1

// Pronóstico 2019 con arch1
quietly arch D.metrobus if tin(2008m1, 2018m12), ar(1) arch(1)
predict fcst_arch1 if tin(2008m1, 2019m12), dynamic(tm(2018m12))
tsline D.metrobus fcst_arch1 if tin(2018m1, 2019m12)


// SERIES MULTIVARIADAS
//------------------------------------------------------------------------------

clear all
use transportep

tsset fecha // Stata se queja de que faltan días

// Con un calendario, le explicamos a Stata que son fines de semana los días que faltan
bcal create calendario, from (fecha) replace
bcal load calendario
generate fecha2 = bofd("calendario", fecha)
format %tbmycal fecha2
tsset fecha2

// Selección de rezagos con varsoc
varsoc d_metro d_metrobus d_trolebus d_rtp, maxlag(5) //Nos sugiere 2 rezagos

// VAR(2)
var d_metro d_metrobus d_trolebus d_rtp, lags(1 2)

// Diapositiva de VAR Bayesiano
bayes: var d_metro d_metrobus d_trolebus d_rtp, lags(1 2)
bayesgraph diagnostics {d_rtp:L.d_metro}

// SVAR(2)
matrix input  A =  ( 1,  0,  0,  0\ ///
					 .,  1,  0,  0\ ///
					 .,  .,  1,  0\ ///
					 .,  .,  .,  1)

matrix input B =   ( .,  0,  0,  0\ ///
                     0,  .,  0,  0\ ///
                     0,  0,  .,  0\ ///
				     0,  0,  0,  .)

svar d_rtp d_trolebus d_metrobus d_metro, aeq(A) beq(B) lags(1 2)

// Funciones de impulso-respuesta
irf set "myIRF", replace
irf create svar1, replace
irf graph oirf